
local _,L = ...
local rematch = Rematch
local frame = RematchFrame
local settings, saved

-- these are layouts for each numbered tab, with "left"/"right" in the index of a rematch.panel to show
-- from Main.lua panels are ordered: PetPanel, LoadoutPanel, TeamPanel, QueuePanel, OptionPanel
frame.layouts = {{"left","right"},{nil,"left","right"},{"left",nil,nil,"right"},{nil,"left",nil,nil,"right"}}
-- layout for narrow mode is just one panel
frame.narrow = {"PetPanel","TeamPanel","QueuePanel","OptionPanel"}

rematch:InitModule(function()
	rematch.Frame = frame
	settings = RematchSettings
	settings.ActivePanel = settings.ActivePanel or 1
	if not settings.CornerPos or not settings.XPos then
		settings.CornerPos = "BOTTOMRIGHT"
		frame:UpdateCorner()
	end
	rematch:SetupPanelTabs(frame.PanelTabs,settings.ActivePanel,L["Pets"],L["Teams"],L["Queue"],L["Options"])
	for i=1,4 do
		frame.PanelTabs.Tabs[i]:SetScript("OnClick",frame.PanelTabOnClick)
	end
	frame.Title:SetText(L["Rematch"])
	frame.NarrowModeButton.tooltipTitle = L["Toggle Single Panel Mode"]
	frame.NarrowModeButton.tooltipBody = L["Toggle between one panel or two panels side by side."]
end)

-- this is UpdateUI stuff, just title and lock button status
-- major changes to frame are done in ConfigureFrame
function frame:Update()
	frame.LockButton:SetChecked(not settings.LockPosition) -- checked is unlocked
end

function frame:Toggle()
	if not InCombatLockdown() then
		frame:SetShown(not frame:IsShown())
	else
		rematch:print("You are in combat. Try again when out of combat.")
	end
end

function frame:OnShow()
	rematch.timeUIChanged = GetTime()
	frame:ConfigureFrame()
	frame.showAfterBattle = nil
	frame.showAfterCombat = nil
	if not settings.LockWindow then
		rematch:SetESCable("RematchFrame",true)
	end
end

function frame:OnHide()
	rematch:HideWidgets()
	rematch:HideDialog()
	rematch:SetESCable("RematchFrame",false)
	if not C_PetBattles.IsInBattle() then
		C_Timer.After(0.75,frame.CheckIfHidingForBattle)
	end
end

function frame:CheckIfHidingForBattle()
	if C_PetBattles.IsInBattle() then
		frame.showAfterBattle = true
	end
end

-- call when frame moves or CornerPos changes (options) to update XPos,YPos to new position
function frame:UpdateCorner()
	local corner = settings.CornerPos
	if corner=="BOTTOMRIGHT" then
		settings.XPos, settings.YPos = frame:GetRight(), frame:GetBottom()
	elseif corner=="TOPRIGHT" then
		settings.XPos, settings.YPos = frame:GetRight(), frame:GetTop()
	elseif corner=="BOTTOMLEFT" then
		settings.XPos, settings.YPos = frame:GetLeft(), frame:GetBottom()
	elseif corner=="TOPLEFT" then
		settings.XPos, settings.YPos = frame:GetLeft(), frame:GetTop()
	end
	frame:SetPoint(corner,UIParent,"BOTTOMLEFT",settings.XPos,settings.YPos)
end

-- this is called in the OnShow, min/maximize and when changing tabs to reconfigure the frame UI
function frame:ConfigureFrame()

	if rematch.Journal:IsVisible() or (PetJournal and PetJournal:IsVisible()) then
		HideUIPanel(CollectionsJournal)
	end

	rematch:HideWidgets()
	rematch:HideDialog()
	rematch:HideNotes()

	-- size frame for minimized or maximized view
	if settings.Minimized then
		frame:SetSize(269,156)
		rematch.MiniPanel:Reparent(frame)
		rematch.Toolbar:Reparent(frame)
		rematch.TeamTabs:Hide()
		rematch.TopPanel:Hide()
		rematch.BottomPanel:Hide()
		rematch.NarrowPanel:Hide()
		rematch.LoadedTeamPanel:Reparent(rematch.MiniPanel)
	elseif settings.NarrowMode then
		frame:SetSize(347,604) -- width would be 290 with panels not changing width
		rematch.Toolbar:Reparent(rematch.NarrowPanel)
		rematch.TeamTabs:Reparent(frame)
		rematch.TeamTabs:SetShown(settings.ActivePanel==2 or settings.AlwaysTeamTabs)
		rematch.TopPanel:Hide()
		rematch.BottomPanel:Hide()
		rematch.NarrowPanel:Show()
	else
		frame:SetSize(572,604)
		rematch.MiniPanel:Hide()
		rematch.Toolbar:Reparent(rematch.TopPanel)
		rematch.TeamTabs:Reparent(frame)
		rematch.TeamTabs:SetShown(settings.ActivePanel==2 or settings.AlwaysTeamTabs)
		rematch.TopPanel:Reparent(frame)
		rematch.TopPanel:Show()
		rematch.BottomPanel:Reparent(frame)
		rematch.BottomPanel:Show()
		rematch.NarrowPanel:Hide()
	end
	frame.MinimizeButton:SetChecked(settings.Minimized and true)


	if not settings.NarrowMode or settings.Minimized then
		-- show/hide panels as defined in layout table
		local layout = frame.layouts[settings.ActivePanel] or frame.layouts[1]
		for i=1,5 do -- only checking first 5 panels
			local panel = _G[rematch.panels[i]]
			if not settings.Minimized and layout[i] and panel then
				panel:SetParent(frame)
				panel:ClearAllPoints()
				panel:SetHeight(520)
				if panel.Resize then
					panel:Resize(280)
				end
				panel:SetPoint("BOTTOMLEFT",layout[i]=="left" and 4 or 286,4+22)
				panel:Show()
			elseif panel then
				panel:Hide()
			end
		end
		-- special handling for queue panel when not minimized
		if not settings.Minimized then
			if settings.ActivePanel==3 then
				rematch.QueuePanel:SetHeight(398)
				rematch.MiniPanel:Reparent(rematch.QueuePanel)
				rematch.LoadedTeamPanel:Reparent(rematch.QueuePanel)
			else
				rematch.LoadedTeamPanel:Reparent(rematch.LoadoutPanel)
			end
		end
	else -- narrow mode view
		rematch.LoadoutPanel:Hide()
		for i=1,4 do
			local panel = rematch[frame.narrow[i]]
			if i==settings.ActivePanel then
				panel:SetParent(frame)
				panel:ClearAllPoints()
				panel:SetHeight(i<4 and 417 or 539) -- 538 for optionpanel, 520 is default panel height
				if panel.Resize then
					panel:Resize(280+57)
				end
				panel:SetPoint("BOTTOMLEFT",4,40)
				panel:Show()
				if i<4 then
					rematch.LoadedTeamPanel:Reparent(panel)
					rematch.MiniPanel:Reparent(panel)
				end
			else
				panel:Hide()
			end
		end
		-- special case for optionpanel (4)
		if settings.ActivePanel==4 then
			rematch.LoadedTeamPanel:Hide()
			rematch.MiniPanel:Hide()
		end
	end

	-- update tabs across the bottom
	rematch:SelectPanelTab(frame.PanelTabs,not settings.Minimized and settings.ActivePanel or 0)
--	rematch:UpdatePanelTabs(frame.PanelTabs)

	-- if a pet is on the cursor, sync the slot glows
	local petID = rematch:GetCursorPet()
	if petID then
		for i=1,3 do
			rematch.LoadoutPanel.Loadouts[i].DropButton.Glow:Stop()
			rematch.LoadoutPanel.Loadouts[i].DropButton.Glow:Play()
		end
		if rematch:PetCanLevel(petID) then
			rematch.QueuePanel.DropButton.Glow:Stop()
			rematch.QueuePanel.DropButton.Glow:Play()
		end
	end

	frame:ClearAllPoints()
	frame:SetPoint(settings.CornerPos,UIParent,"BOTTOMLEFT",settings.XPos,settings.YPos)

	-- finally do an UpdateUI to update the contents of all shown panels
	rematch:UpdateUI()
end

function frame:SelectPanel(index,unselect)
	settings.ActivePanel = index
	frame:ConfigureFrame()
end

function frame:MoveStart()
	if not settings.LockPosition or IsShiftKeyDown() then
		rematch:HideWidgets()
		frame:StartMoving()
	end
end

function frame:MoveStop()
	frame:StopMovingOrSizing()
	frame:SetUserPlaced(false)
	frame:UpdateCorner()
end

-- onclick of the minimize button
function frame:ToggleSize()
	settings.Minimized = not settings.Minimized
	rematch:HideWidgets()
	rematch.timeUIChanged = GetTime()
	frame:ConfigureFrame()
end

function frame:LockButtonOnClick()
	settings.LockPosition = not settings.LockPosition
	frame:Update()
end

function frame:MinimizeOnKeyDown(key)
	if key==GetBindingKey("TOGGLEGAMEMENU") and not settings.Minimized then
		frame:ToggleSize() -- minimize but don't pass ESC along
		self:SetPropagateKeyboardInput(false)
	else -- ESC not hit (or already minimized), send it along
		self:SetPropagateKeyboardInput(true)
	end
end

-- when one of the panel tabs is clicked
function frame:PanelTabOnClick()
	if settings.Minimized then
		settings.Minimized = nil
		RematchFrame:SelectPanel(self:GetID())
	elseif settings.ActivePanel==self:GetID() then
		-- if we're clicking the same tab, then minimize
		frame:ToggleSize()
	else -- otherwise select new tab
		frame:SelectPanel(self:GetID())
	end
end
