local _,L = ...
local rematch = Rematch
local panel = RematchLoadedTeamPanel

rematch:InitModule(function()
	rematch.LoadedTeamPanel = panel
	panel:RegisterForClicks("AnyUp")
	panel.tooltipTitle = L["Last Loaded Team"]
	panel.tooltipBody = format("%s %s\n%s %s",rematch.LMB,L["Reload Team"],rematch.RMB,L["Team Options"])
end)

-- reparents and positions this panel to parent
function panel:Reparent(parent)
	panel:SetParent(parent)
	-- this panel always spans whole bottom of parent frame or journal
	panel:ClearAllPoints()
	if parent==rematch.MiniPanel then
		panel:SetPoint("TOPLEFT",rematch.Frame,"TOPLEFT",4,-24)
		panel.maxWidth = 260
	elseif parent==rematch.LoadoutPanel then
		panel:SetPoint("TOPLEFT",rematch.LoadoutPanel,"TOPLEFT",0,-90)
		panel.maxWidth = 280
	else -- for all of the standalone panels
		if RematchSettings.NarrowMode then
			panel:SetPoint("TOPLEFT",rematch.Frame,"TOPLEFT",4,-24)
		else
			panel:SetPoint("BOTTOMLEFT",parent,"TOPLEFT",0,96)
		end
		-- if parenting to a panel in narrow mode, it's the wide 337; otherwise 280 above queue
		panel.maxWidth = RematchSettings.NarrowMode and 337 or 280
	end
	panel:Show()
end

function panel:Update()
	local settings = RematchSettings
	local key = settings.loadedTeam
	if key then
		panel.Name:SetText(rematch:GetTeamTitle(settings.loadedTeam,true))
	else
		panel.Name:SetText(BATTLE_PET_SLOTS)
	end
	panel.key = key
	panel.Footnotes.key = key
	local team = RematchSaved[key]
	local showNotes = team and team.notes and true
	local showPreferences = rematch:HasPreferences(team)
	panel.Footnotes.Notes:SetShown(showNotes)
	panel.Footnotes.Preferences:SetShown(showPreferences)
	local panelWidth = panel.maxWidth or 280
	local footnoteWidth = (showNotes and showPreferences) and 51 or (showNotes or showPreferences) and 29 or 0
	if footnoteWidth>0 then
		panel.Footnotes:SetWidth(footnoteWidth)
		panel.Footnotes:Show()
		panel:SetWidth(panelWidth-footnoteWidth-2)
	else
		panel.Footnotes:Hide()
		panel:SetWidth(panelWidth)
	end
end

function panel:OnClick(button)
	if button=="RightButton" and self.key then
		rematch:SetMenuSubject(self.key)
		rematch:ShowMenu("TeamMenu","cursor")
	elseif self.key then
		rematch:LoadTeam(self.key)
	end
end